using System;
using System.Data.Entity;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using Ninject;
using Ninject.Modules;
using TourDataManager.Entities;

namespace TourDataManager {

    
    /// <summary>
    /// Фасад для всего плагина
    /// </summary>
    public class TourDataManager {

        private static string PersistentPath;

        public static string GetPathInPersistent(string relative){
            return Path.Combine(PersistentPath, relative);
        }

        public FetchEstatesUseCase estatesfetch{ get; set; }
        public FetchTourPreviewsUseCase tourpreviewsfetch{ get; set; }
        public Db database{ get; set; }


        private IKernel Container;
        
        public TourDataManager(string persistentPath){
            PersistentPath = persistentPath;
            Container = new StandardKernel(new MyModule());
            
            
            estatesfetch = Container.Get<FetchEstatesUseCase>();
            tourpreviewsfetch = Container.Get<FetchTourPreviewsUseCase>();
            database = Container.Get<Db>();
        }

        public void Login(string email, string password){
            Container.Get<IAuthenticator>().Login(email,password, (b, s) => {
                var cookiestor = Container.Get<CookieStorage>();
                Debug.Log($"Authorization : {s}");
                Debug.Log($"Cookie count in storage : {cookiestor.Get().Count}");
                cookiestor.Save();
            });
        }

        public async Task<Estate[]> DownloadEstates(){
            var estates = await estatesfetch.FetchEstatesFromServerAsync();
            database.InsertEstates(estates);
            return estates;
        }

        public async Task<Tour[]> DownloadTourPreviews(long estateId){
            var tours = await tourpreviewsfetch.FetchTourFromServerAsync(estateId);
            database.InsertTours(tours);
            return tours;
        }

        public ContentLoadingTask DownloadTourContent(long tourId){
            return Container.Get<FetchTourContentUseCase>().FetchTourContentFromServerAsync(tourId);
        }
    }

    public class MyModule : NinjectModule {


        public override void Load(){
            var cookieStorage = new CookieStorage(TourDataManager.GetPathInPersistent("cookie.storage"));
            var httpClientHandler = new HttpClientHandler();
            //CookieContainer сам запоминает полученные из ответа куки
            httpClientHandler.CookieContainer = cookieStorage.Get();
            var httpClient = new HttpClient(httpClientHandler);
            
            
            Bind<CookieStorage>().ToConstant(cookieStorage).InSingletonScope();
            Bind<HttpClientHandler>().ToConstant(httpClientHandler).InSingletonScope();
            Bind<HttpClient>().ToConstant(httpClient).InSingletonScope();
            Bind<IAuthenticator>().To<Authenticator>().InSingletonScope();
            Bind<FetchEstatesUseCase>().ToSelf().InSingletonScope();
            Bind<FetchTourPreviewsUseCase>().ToSelf().InSingletonScope();
            Bind<FetchTourContentUseCase>().ToSelf().InSingletonScope();
            Bind<Db>().ToSelf().InSingletonScope();

        }
    }
}